float Script : STANDARDSGLOBAL <
    string ScriptOutput = "color";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
> = 0.8;

float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;	
float Scale : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;

static float Si=Scale*0.1;
float LengthMax = 1.0;
static float BlurScale = Si;
static float BlurScale2 = Si;
float2 ViewportSize : VIEWPORTPIXELSIZE;

static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize);
static float2 SampStep = (float2(0.5,0.5) / ViewportSize)*2;

float3	LightPosition	: POSITION  < string Object = "Light"; >;
float3	LightDirection	: DIRECTION < string Object = "Light"; >;
float3	LightDiffuse	: DIFFUSE   < string Object = "Light"; >;
float3	LightAmbient	: AMBIENT   < string Object = "Light"; >;

float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection   : DIRECTION < string Object = "Camera"; >;
float4x4 WorldViewProjMatrix      : WORLDVIEWPROJECTION;
float4x4 ViewProjMatrix           : VIEWPROJECTION;

#define SAMPNUM 4

float ClearDepth  = 1.0;
float4 ClearColor = float4(0, 0, 0, 1);


shared texture2D NormalDepthKrMap : RENDERCOLORTARGET ;
sampler2D NormalDepthSamp = sampler_state {
	texture = <NormalDepthKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

shared texture2D AlbedoMap : RENDERCOLORTARGET;
sampler2D AlbedoSamp = sampler_state {
	texture = <AlbedoMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};


texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D ScnSamp = sampler_state {
    texture = <ScnMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};


////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw( float4 Pos : POSITION, float4 Tex : TEXCOORD0 ) {
    VS_OUTPUT Out = (VS_OUTPUT)0; 
    
    Out.Pos = Pos;
    Out.Tex = Tex + ViewportOffset;
    return Out;
}

float4 PS_passDraw2(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(ScnSamp, Tex);
	float2 SampStepRate = SampStep;
	float4 Depth = abs(tex2D(NormalDepthSamp, Tex).a);

	float3 Albedo = tex2D(AlbedoSamp, Tex);
	float3 AlbedoX = tex2D(AlbedoSamp, Tex + float2(-SampStepRate.x, 0.0));
	float3 AlbedoXm = tex2D(AlbedoSamp, Tex + float2(SampStepRate.x, 0.0));
	float3 AlbedoY = tex2D(AlbedoSamp, Tex + float2(0.0,SampStepRate.y));
	float3 AlbedoYm = tex2D(AlbedoSamp, Tex + float2(0.0,-SampStepRate.y));

	float3 AlbedoXY = tex2D(AlbedoSamp, Tex + float2(SampStepRate.x, SampStepRate.y));
	float3 AlbedoXYm = tex2D(AlbedoSamp, Tex + float2(SampStepRate.x, -SampStepRate.y));
	float3 AlbedoXmY = tex2D(AlbedoSamp, Tex + float2(-SampStepRate.x, SampStepRate.y));
	float3 AlbedoXmYm = tex2D(AlbedoSamp, Tex + float2(-SampStepRate.x, -SampStepRate.y));

	float RFlag=max(distance(Albedo,AlbedoX),	
		max(distance(Albedo,AlbedoY),
		max(distance(Albedo,AlbedoXm),
		max(distance(Albedo,AlbedoYm),
		max(distance(Albedo,AlbedoXY),
		max(distance(Albedo,AlbedoXYm),
		max(distance(Albedo,AlbedoXmY),distance(Albedo,AlbedoXmYm))))))));

	RFlag=saturate(1.0-RFlag*10);

	float4 NormalS = tex2D(NormalDepthSamp, Tex);
	float4 NormalSX = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0));
	float4 NormalSXm = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0));
	float4 NormalSY = tex2D(NormalDepthSamp, Tex + float2(0.0,SampStepRate.y));
	float4 NormalSYm = tex2D(NormalDepthSamp, Tex + float2(0.0,-SampStepRate.y));

	float4 NormalSXY = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, SampStepRate.y));
	float4 NormalSXYm = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, -SampStepRate.y));
	float4 NormalSXmY = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, SampStepRate.y));
	float4 NormalSXmYm = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, -SampStepRate.y));

	float NFlag=max(distance(NormalS.xyz,NormalSX.xyz),	
		max(distance(NormalS.xyz,NormalSY.xyz),
		max(distance(NormalS.xyz,NormalSXm.xyz),
		max(distance(NormalS.xyz,NormalSYm.xyz),
		max(distance(NormalS.xyz,NormalSXY.xyz),
		max(distance(NormalS.xyz,NormalSXYm.xyz),
		max(distance(NormalS.xyz,NormalSXmY.xyz),distance(NormalS.xyz,NormalSXmYm.xyz))))))));

	NFlag=saturate(1.0-NFlag*4);


	float2 SampStepRateS = SampStep*(RFlag*NFlag)*Si*saturate(1.0-Depth*0.03);
	float3 BColor=0;
	[unroll]
	for (int ix =-SAMPNUM; ix <= SAMPNUM; ix++) {
	for (int iy = -SAMPNUM; iy <= SAMPNUM; iy++) {
		float2 AddTexB = float2(ix,iy);
		float2 Stex = Tex + SampStepRateS*AddTexB;
		
		BColor += tex2D(ScnSamp, Stex).rgb;

	}
	}
	Color.rgb=BColor/pow(SAMPNUM*2+1,2);


	return Color;
}

////////////////////////////////////////////////////////////////////////////////////////////////
technique AA <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"

	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw2;"
	;


> {
	pass Draw2 < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw2();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
