float Script : STANDARDSGLOBAL <
    string ScriptOutput = "color";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
> = 0.8;

float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;	
float Scale : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;
float X : CONTROLOBJECT < string name = "(self)"; string item = "X"; >;
float Y : CONTROLOBJECT < string name = "(self)"; string item = "Y"; >;

static float Si=Scale*0.1;
float LengthMax = 1.0;
static float BlurScale = Si;
static float BlurScale1 = Si*0.25*(X+1.0);
static float BlurScale2 = Si*0.25*(Y+1.0);
float2 ViewportSize : VIEWPORTPIXELSIZE;

static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize);
static float2 SampStep = (float2(0.5,0.5) / ViewportSize)*2*BlurScale;

float3	LightPosition	: POSITION  < string Object = "Light"; >;
float3	LightDirection	: DIRECTION < string Object = "Light"; >;
float3	LightDiffuse	: DIFFUSE   < string Object = "Light"; >;
float3	LightAmbient	: AMBIENT   < string Object = "Light"; >;

float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection   : DIRECTION < string Object = "Camera"; >;
float4x4 WorldViewProjMatrix      : WORLDVIEWPROJECTION;
float4x4 ViewProjMatrix           : VIEWPROJECTION;

#define SAMPNUM 64

float ClearDepth  = 1.0;
float4 ClearColor = float4(0, 0, 0, 1);


shared texture2D WPosKrMap : RENDERCOLORTARGET ;
sampler2D WPosSamp = sampler_state {
	texture = <WPosKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = MIRROR;
	AddressV = MIRROR;
};
shared texture2D NormalDepthKrMap : RENDERCOLORTARGET ;
sampler2D NormalDepthSamp = sampler_state {
	texture = <NormalDepthKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};



texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D ScnSamp = sampler_state {
    texture = <ScnMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};
texture2D ScnOldMap : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
string Format = "A16B16G16R16F";
>;
sampler2D ScnOldSamp = sampler_state {
    texture = <ScnOldMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};


////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw( float4 Pos : POSITION, float4 Tex : TEXCOORD0 ) {
    VS_OUTPUT Out = (VS_OUTPUT)0; 
    
    Out.Pos = Pos;
    Out.Tex = Tex + ViewportOffset;
    return Out;
}


float4 PS_passDrawZ(float2 Tex: TEXCOORD0) : COLOR
{

	float4 Color = tex2D(ScnSamp, Tex);
	float4 WPos = tex2D(WPosSamp, Tex);
	float2 SampStepRate = SampStep;

	float4 ColorSampX0D = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	float4 ColorSampY0D = tex2D(NormalDepthSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	float4 ColorSampY2D = tex2D(NormalDepthSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	float4 ColorSampX2D = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;

	float EdgeRim=10.0;
	float4 XEdgeND = min(EdgeRim,max(-EdgeRim,ColorSampX2D - ColorSampX0D));
	float4 YEdgeND = min(EdgeRim,max(-EdgeRim,ColorSampY2D - ColorSampY0D));



	float4  WPosOld= tex2D(ScnOldSamp, Tex);


	float BPower=min(1.0,distance(WPos.rgb,WPosOld.rgb)*0.1);

	float2 EdgeRate = float2(-YEdgeND.a, XEdgeND.a)*BPower*0.1*BlurScale;


	float2 SampStepRateS=SampStep*EdgeRate;
	float3 BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateS;
		BlurColor += tex2D(ScnSamp, Tex + SampStepTex).rgb * e;
		n += e;
	}
	Color.rgb = BlurColor / n;

	return Color;
}

float4 PS_passDraw2(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(WPosSamp, Tex);


	//Color.a=length(CameraPosition+CameraDirection*100);
	return Color;
}

////////////////////////////////////////////////////////////////////////////////////////////////
technique AA <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"
	
	
	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=DrawZ;"


	"RenderColorTarget0=ScnOldMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw2;"
	;


> {
	pass DrawZ < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDrawZ();
	}
	pass Draw2 < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw2();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
